
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Overview of Code (12 March 2020)
%
%  1. Input data moments
%  2. Set up grids for savings, wealth, and % of work in outside markets
%  3. Loop over parameter values until target moments are matched,
%  computing the stationary equilibrium of the model in each iteration.
%    3a. Update parameter values based on results of last iteration
%    3b. Compute value functions and decision rules in inner period problem
%    3c. Compute value functions and decision rules in outer period problem
%    3d. Compute stationary distribution of assets
%    3e. Compute model moments
%  4. Compute the transition and new stationary equilibrium after bridge
%  5. Compute change in only mean of tau
%  6. Compute change in only sd of tau
%  7. Compute model output tables
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Input data moments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Inside_Earn_share_pre = 0.6646;
Perc_Both = 0.2237;
Pre_Premium = 1.3716;
Post_Premium = 1.0114;
Intm_Spend_data = 0.8830;
Intm_Spend_change = 1.7023;
SD_Log_Harvest = 1.3057;

tau = .545;
tau1=tau;
pi = .41;       % Probability of flood when flood risk is present
T = 26;         % Total number of inner periods
Tdry = 13;      % Number of inner periods without flooding risk
Twet = T-Tdry;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  2. Set up grids for savings, wealth, and % of work in outside markets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sN = 500;
sgrid = .01*1.025.^[0:sN-1]-.01;
oN = 10; % Number of grid points on omega grid
omega_grid = 1/2/oN+[0:oN-1]./oN;


% Solve for within-season value function and optimal labor allocation
Wo = zeros(sN,oN);
Wb = zeros(sN,oN);

mu = 0.8;
Px = 1;

AN = 200;
ANf = AN;
NNN = 10000;
Agrid = 0.0001*1.12.^[0:AN-1];
Agridf = Agrid;
g = (Agrid(AN)/Agrid(1))^(1/(NNN-1));
Agrid_int = Agrid(1)*g.^[0:NNN-1];

% Setting some parameter values and initial guesses
b = 0.96;
s =  4.397;
zm = 2.759;
zs = 0.915;
fo = 0.0179;
eta = 3.191;
a = 7.38*10^-4;
fi = 1 - Post_Premium*(1-fo);
cbar = 0.0;

wo = 1;
wi = 1/Pre_Premium;
wi2 = 1/Post_Premium;

% Initial guess of value function and measure of assets
M = zeros(ANf,1); % Measure of househols at each grid point in the asset distribution
M(2:ANf-1) = 1/(ANf-2);

V2 = zeros(AN,1);
M2 = M;

saveopt = ceil([1:AN]/AN*15); % Initial guess at optimal savings rule

% These are values that assist in the efficient updating of parameter values.
Total_Inside_pre = .72;
Inside_Labor_post_model = .465;

%load('matrices.mat')
iter = 0;
RMSE = 100;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  3. Loop over parameter values until target moments are matched,
%  computing the stationary equilibrium of the model in each iteration.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

while RMSE >.001
    
    % Parameters of the updating processes
    max_change = .1;
    update = .1;
    
    if iter>2
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %    3a. Update parameter values based on results of last iteration
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Total_Inside_pre = max(.5,min(.8,Total_Inside_pre_exact));
        zm = (1-update)*zm + update*zm*max(1-max_change,min(1+max_change,1/Excess_Labor_Demand));
        zs = (1-update)*zs + update*zs*max(1-max_change,min(1+max_change,DATA(4)/MODEL(4)));
        s = (1-update)*s + max(1-max_change,min(1+max_change,DATAu(1)/MODELu(1)))*update*s;
        fo = (1-update)*fo + update*max(1-max_change,min(1+max_change,(1+MODELu(2))/(1+DATAu(2))))*fo;
        eta = max(2.5,min(3.3,((1-update)*eta + update*(log(Intm_Spend_change)-log(Inside_Labor_post_model/Total_Inside_pre))/log(Pre_Premium/Post_Premium))));
        a = min(.99999,max(10^-5,1/(1+(Twet*wi)^eta*Total_Inside_pre/(Intm_Spend_data*sum((Inside_Earn+Outside_Earn).*M)))));
        
    end
    wi = 1/Pre_Premium;
    
    iter = iter + 1
    tic
    
    iteru = 0;
    while iteru<5
        iteru = iteru+1;
        fi = 1 - Post_Premium*(1-fo);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %    3b. Compute value functions and decision rules in inner period problem
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Wb_temp = zeros(sN,oN); % Compute the inner period value function for every savings rate and omega value
        for kk = 1:oN % Iterating over omega levels
            Wbb = zeros(sN,T);
            in_share = omega_grid(kk);
            Wbb(:,T) = pi*(sgrid'+(in_share*wi+(1-in_share)*wo*(1-tau))*(1-fo-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wi+(1-in_share)*wo)*(1-fo-fi)-cbar).^(1-s)/(1-s);
            for jj = 1:T-Tdry-1 % Iterating backward through time
                Wbb(:,T-jj) = pi*max(( (subplus((in_share*wi+(1-in_share)*wo*(1-tau))*(1-fo-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wbb(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wi+(1-in_share)*wo)*(1-fo-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wbb(:,T-jj+1)')');
            end
            for jj = T-Tdry:T-1
                Wbb(:,T-jj) = max( (subplus(wo*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wbb(:,T-jj+1)')');
                
            end
            Wb_temp(:,kk) = Wbb(:,1); % Record the value function in the first period, which is the value from the perspective of the beginning of the year
        end
        omega_s = zeros(1,sN);
        Wb = zeros(1,sN);
        for jj = 1:sN % Choose the optimal omega level for each level of savings
            [Wb(jj),oopt] = max(Wb_temp(jj,:)');
            omega_s(jj) = omega_grid(oopt);
        end
        
        Woo = zeros(sN,T); % Compute the value function conditional on working only in outside labor market
        in_share = 0;
        Woo(:,T) = pi*(sgrid'+(in_share*wi+(1-in_share)*wo*(1-tau))*(1-fo)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wi+(1-in_share)*wo)*(1-fo)-cbar).^(1-s)/(1-s);
        for jj = 1:T-Tdry-1
            Woo(:,T-jj) = pi*max(( (subplus((in_share*wi+(1-in_share)*wo*(1-tau))*(1-fo)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Woo(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wi+(1-in_share)*wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Woo(:,T-jj+1)')');
        end
        for jj = T-Tdry:T-1
            Woo(:,T-jj) = max( (subplus((in_share*wo+(1-in_share)*wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Woo(:,T-jj+1)')');
            
        end
        Wo = Woo(:,1)';
        
        
        Wii = zeros(sN,T); % Compute the value function conditional on working only in inside labor market
        in_share = 1;
        Wii(:,T) = pi*(sgrid'+(in_share*wi+(1-in_share)*wo*(1-tau))*(1-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wi+(1-in_share)*wo)*(1-fi)-cbar).^(1-s)/(1-s);
        for jj = 1:T-Tdry-1
            Wii(:,T-jj) = pi*max(( (subplus((in_share*wi+(1-in_share)*wo*(1-tau))*(1-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wii(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wi+(1-in_share)*wo)*(1-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
        end
        for jj = T-Tdry:T-1
            Wii(:,T-jj) = max( (subplus((wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
        end
        Wi = Wii(:,1)';
        
        % Record which of the three options they choose, conditional on savings
        Inside = [Wi>Wb].*[Wi>Wo];
        Both = [Wi<Wb].*[Wb>Wo];
        Outside = [Wo>=Wb].*[Wi<Wo];
        
        W = max(max(Wb,Wi),Wo); % Inner value function is the max over three discrete options
        
        % Use a spline to interpolate savings rules over the asset distribution
        save_imp = spline(Agrid,sgrid(saveopt),Agrid_int);
        WWi = spline(sgrid,Wi,save_imp);
        WWo = spline(sgrid,Wo,save_imp);
        WWb = spline(sgrid,Wb,save_imp);
        Mm = spline(Agrid,M,Agrid_int);
        Mm = Mm/sum(Mm);
        
        Frac_Both = [WWb>max(WWi,WWo)]*Mm'; % Percentage of households working in both markets
        
        Inside_Labor_pre = zeros(ANf,1);
        Inside_Earn = zeros(ANf,1);
        Outside_Earn = zeros(ANf,1);
        saveoptf = saveopt;
        for aa = 1:ANf
            Inside_Labor_pre(aa) = [Inside(saveoptf(aa))==1]*(1-fi) + [Both(saveoptf(aa))==1]*omega_s(saveoptf(aa))*(1-fi-fo);
            Inside_Earn(aa) = wi*(1-fi)*[Inside(saveoptf(aa))==1] + wi*(1-fi-fo)*[Both(saveoptf(aa))==1]*omega_s(saveoptf(aa));
            Outside_Earn(aa) = wo*(1-fo)*[Outside(saveoptf(aa))==1] + wo*(1-fi-fo)*[Both(saveoptf(aa))==1]*(1-omega_s(saveoptf(aa)));
        end
        
        Labor_Supply = sum(Inside_Labor_pre.*M);
        
        % These two moments are mainly updated by parameters of the inner period
        % value function, so it is efficient to improve on them independent of the
        % outer period value function updating
        MODELu = [1/(1+(Outside_Earn'*M)/(Inside_Earn'*M)) Frac_Both  ];
        DATAu = [Inside_Earn_share_pre Perc_Both ];
        
        disp([s fo])
        RMSEu = (sum((MODELu./DATAu-1).^2)/2)^.5 % RMSE of just these two moments
        if RMSEu>.01 % Independent updating of these two parameters
            s = (1-update)*s + max(1-max_change,min(1+max_change,DATAu(1)/MODELu(1)))*update*s;
            fo = (1-update)*fo + update*max(1-max_change,min(1+max_change,(1+MODELu(2))/(1+DATAu(2))))*fo;
        else
            iteru=100;
        end
    end
    
    % Since updating on the value function is a bottleneck, only doing it every
    % 5th iteration. Correspondingly, the RMSE used to evaluate the while loop
    % is only changed on those iterations.
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %    3c. Compute value functions and decision rules in outer period problem
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if mod(iter,5) == 1
        if iter == 1
            V = ones(1,AN)*(Agrid(AN/2)^(1-s)-1)/(1-s);
            Vp = V;
        end
        
        Vdist = 10;
        while Vdist >.005
            Continuation = -10^16*[(Agrid'*ones(1,sN)-ones(AN,1)*sgrid)<0];
            Continuation = Continuation + V(1)*(.5+.5*erf(((log(Agrid(1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
            for aap = 2:AN-1
                Continuation = Continuation + V(aap)*((.5+.5*erf(((log(Agrid(aap))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agrid(aap-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
            end
            Continuation = Continuation + V(AN)*(1-(.5+.5*erf(((log(Agrid(AN-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
            
            
            [Vp,saveopt] = max( ((1-b)*ones(AN,1)*W + b*Continuation)' );
            
            Vdist = sum(abs(Vp-V))/AN;
            V = Vp;
        end
        
        saveoptf = saveopt;
    end
    
    % Compute model moments used later
    Total_sell_model = 0;
    for aa = 1:ANf
        Total_sell_model = Total_sell_model + sgrid(saveoptf(aa))*M(aa);
    end
    Total_sell_model = 1-Total_sell_model/(Agridf*M);
    
    Ni = zeros(1,ANf);
    omega = zeros(ANf,1);
    for aa = 1:ANf
        Ni(1,aa) = (Agridf(aa)-sgrid(saveoptf(aa)))/Twet/wi/(1+a/(1-a)*(Twet*wi/Px)^(eta-1));
        omega(aa) = omega_s(saveoptf(aa));
    end
    Xi = a/(1-a)*(Twet*wi/Px)^(eta)*Ni;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %    3d. Compute stationary distribution of assets
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    distM = 10;
    while distM > .001
        Mp = zeros(ANf,1);
        for aa = 1:ANf
            Mp(1) = Mp(1) + M(aa)*(.5+.5*erf(((log(Agridf(1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptf(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
            
            for aap = 2:ANf-1
                Mp(aap) = Mp(aap) + M(aa)*((.5+.5*erf(((log(Agridf(aap))-mu*log(subplus(Agridf(aa)-sgrid(saveoptf(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agridf(aap-1))-mu*log((Agridf(aa)-sgrid(saveoptf(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
            end
            Mp(ANf) = Mp(ANf) + M(aa)*(1-(.5+.5*erf(((log(Agridf(ANf-1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptf(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
        end
        
        distM = sum(abs(Mp-M));
        M = Mp/sum(Mp);
    end
    
    % Compute value function in the first period of the transition
    Wii = zeros(sN,T);
    in_share = 1;
    Wii(:,T) = pi*(sgrid'+(in_share*wi2+(1-in_share)*wo*(1-0))*(1-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wi2+(1-in_share)*wo)*(1-fi)-cbar).^(1-s)/(1-s);
    for jj = 1:T-Tdry-1
        Wii(:,T-jj) = pi*max(( (subplus((in_share*wi2+(1-in_share)*wo*(1-0))*(1-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wii(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wi2+(1-in_share)*wo)*(1-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
    end
    for jj = T-Tdry:T-1
        Wii(:,T-jj) = max( (subplus((wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
    end
    Wi2 = Wii(:,1)';
    
    W2 = Wi2;
    
    max_change = .1;
    
    % Compute whole transition. Only done every 10 iterations. This is greatly
    % simplified by the fact that the value of being an inside or outside
    % worker are equal, and no one does both in equilibrium.
    if mod(iter,10)==1
        if iter == 1
            V2 = V + 2.1;
            Vp = V + 2.1;
        end
        
        Vdist = 10;
        while Vdist >.005
            Continuation = -10^16*[(Agrid'*ones(1,sN)-ones(AN,1)*sgrid)<0];
            Continuation = Continuation + V2(1)*(.5+.5*erf(((log(Agrid(1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi2)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
            for aap = 2:AN-1
                Continuation = Continuation + V2(aap)*((.5+.5*erf(((log(Agrid(aap))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi2)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agrid(aap-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi2)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
            end
            Continuation = Continuation + V2(AN)*(1-(.5+.5*erf(((log(Agrid(AN-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wi2)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
            
            
            [Vp,saveoptf2] = max( ((1-b)*ones(AN,1)*W2 + b*Continuation)' );
            
            Vdist = sum(abs(Vp-V2))/AN;
            V2 = Vp;
        end
    end
    Ni2 = zeros(1,ANf);
    for aa = 1:ANf
        Ni2(1,aa) = (Agridf(aa)-sgrid(saveoptf2(aa)))/Twet/wi2/(1+a/(1-a)*(Twet*wi2/Px)^(eta-1));
    end
    Xi2 = a/(1-a)*(Twet*wi2/Px)^(eta)*Ni2;
    
    Total_sell_model2 = 0;
    for aa = 1:ANf
        Total_sell_model2 = Total_sell_model2 + sgrid(saveoptf2(aa))*M(aa);
    end
    Total_sell_model2 = 1-Total_sell_model2/(Agridf*M);
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %    3e. Compute model moments
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for aa = 1:ANf
        Inside_Labor_pre(aa) = [Inside(saveoptf(aa))==1]*(1-fi) + [Both(saveoptf(aa))==1]*omega_s(saveoptf(aa))*(1-fi-fo);
        Outside_Labor_pre(aa) = [Outside(saveoptf(aa))==1]*(1-fo) + [Both(saveoptf(aa))==1]*(1-omega_s(saveoptf(aa)))*(1-fi-fo);
        Inside_Earn(aa) = wi*(1-fi)*[Inside(saveoptf(aa))==1] + wi*(1-fi-fo)*[Both(saveoptf(aa))==1]*omega_s(saveoptf(aa));
        Outside_Earn(aa) = wo*(1-fo)*[Outside(saveoptf(aa))==1] + wo*(1-fi-fo)*[Both(saveoptf(aa))==1]*(1-omega_s(saveoptf(aa)));
    end
    
    Labor_Supply = sum(Inside_Labor_pre.*M);
    
    Inside_Labor_post_model = min(.6,max(.3,Ni2*M));
    Intm_Spending = (Xi*M)/sum((Inside_Earn+Outside_Earn).*M);
    
    Total_Inside_pre_exact = sum(Inside_Labor_pre.*M);
    
    Excess_Labor_Demand = Ni*M / max(.001,Labor_Supply);
    SD_Log_Harvest_model = (  sum((log(Agridf).^2)*M) - sum(log(Agridf)*M)^2  )^.5;
    
    MODEL = [MODELu(2) MODELu(1) Excess_Labor_Demand SD_Log_Harvest_model (Xi2*M)/(Xi*M)-1 Intm_Spending 1/wi2];
    DATA = [DATAu(2) DATAu(1) 1 SD_Log_Harvest Intm_Spend_change-1 Intm_Spend_data 1/wi2];
    
    disp([s fo zm zs eta])
    Diff = MODEL./DATA-1 % Raw difference between model and moment values
    if mod(iter,5) == 1
        RMSE = (1/length(Diff)*sum(Diff.^2)).^.5 % Root mean squared difference (in percentage) between model and data moments
    end
    toc
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  4. Compute the transition and new stationary equilibrium after bridge
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Transition = 50; % Maximum transition period in years

% Transition in M; notice that since wages are fixed by no arbitrage, there is no
% transition in prices or value functions

% Computing the measure of assets period-by-period over the transition.
% Notice that after the bridge, the only endogenous price (wi2) is set by
% an indifference condition and constant over time. Therefore the
% transition is totally characterized by the change the distribution of
% assets.

Mtt = zeros(AN,Transition);
Mtt(:,1) = M;
Consumption_trans = zeros(1,Transition);

for tt = 1:Transition-1
    for aa = 1:ANf
        Mtt(1,tt+1) = Mtt(1,tt+1) + + Mtt(aa,tt)*(.5+.5*erf(((log(Agridf(1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptf2(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
        for aap = 2:ANf-1
            Mtt(aap,tt+1) = Mtt(aap,tt+1) + Mtt(aa,tt)*((.5+.5*erf(((log(Agridf(aap))-mu*log(subplus(Agridf(aa)-sgrid(saveoptf2(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agridf(aap-1))-mu*log((Agridf(aa)-sgrid(saveoptf2(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
        end
        Mtt(ANf,tt+1) = Mtt(ANf,tt+1) + Mtt(aa,tt)*(1-(.5+.5*erf(((log(Agridf(ANf-1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptf2(aa))))-mu/(eta-1)*log((1-a)/(Twet*wi)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
        Mtt(:,tt+1)=Mtt(:,tt+1)/sum(Mtt(:,tt+1));
    end
    Consumption_trans(1,tt) = (sgrid(saveoptf2)+Twet*wi2*(1-fi)+Tdry*wo*(1-fo))*Mtt(:,tt);
end
Consumption_trans(1,Transition) = (sgrid(saveoptf2)+Twet*wi2*(1-fi)+Tdry*wo*(1-fo))*Mtt(:,Transition);
Consumption_pre = (sgrid(saveoptf)+Twet*Inside_Earn'+Twet*Outside_Earn'*(1-pi+pi*(1-tau))+Tdry*wo*(1-fo))*M;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  5. Compute change in only mean of tau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Notice that the economy starts at the same initial stationary equilibrium
% as before, the only difference is the transition path and new stationary
% equilibrium.

wom = 1.225;
taum = .444;
wim = wom*wi/wo;
tic

% Outside Labor Value
Woo = zeros(sN,T);
in_share = 0;
Woo(:,T) = pi*(sgrid'+(in_share*wim+(1-in_share)*wom*(1-taum))*(1-fo)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wim+(1-in_share)*wom)*(1-fo)-cbar).^(1-s)/(1-s);
for jj = 1:T-Tdry-1
    Woo(:,T-jj) = pi*max(( (subplus((in_share*wim+(1-in_share)*wom*(1-taum))*(1-fo)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Woo(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wim+(1-in_share)*wom)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Woo(:,T-jj+1)')');
end
for jj = T-Tdry:T-1
    Woo(:,T-jj) = max( (subplus((in_share*wo+(1-in_share)*wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Woo(:,T-jj+1)')');
end
Wom = Woo(:,1)';

% Initial guess: wim makes the savings = 0 household indifferent between
% inside and outside. If labor supply >= labor demand at that wim, this is
% an equilibrium.
wim = ((Wom(1)-Tdry*(1-fo)*wo^(1-s)/(1-s))*(1-s)/Twet)^(1/(1-s));

tol1 = .005;
diff1 = 1;
while diff1 > tol1
    Wii = zeros(sN,T);
    in_share = 1;
    Wii(:,T) = pi*(sgrid'+(in_share*wim+(1-in_share)*wom*(1-taum))*(1-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wim+(1-in_share)*wom)*(1-fi)-cbar).^(1-s)/(1-s);
    for jj = 1:T-Tdry-1
        Wii(:,T-jj) = pi*max(( (subplus((in_share*wim+(1-in_share)*wom*(1-taum))*(1-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wii(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wim+(1-in_share)*wom)*(1-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
    end
    for jj = T-Tdry:T-1
        Wii(:,T-jj) = max( (subplus((wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
    end
    Wim = Wii(:,1)';
    
    diff1 = abs(Wim(1)-Wom(1)) + .1*[Wim(1)<Wom(1)]
    if diff1>tol1
        wim = wim + rand(1,1)*max(-.01,min(.01,(Wom(1)-Wim(1))*abs(Wim(1)-Wom(1))))+.01*[Wim(1)<Wom(1)]
    end
end

wimL = wim; % This is a lower bound on the wage; else everyone (even those with zero savings) strictly prefer to work outside.

Vm = log(wim/wi) + V;
Mm = M;

% Start with the new steady state

% Unlike in the baseline case, since there is still risk after the change
% in this economy, we do have the compute new inside wages and labor market
% choices in every period. However, as discussed before, this is made
% easier by the fact that the inside wage has a lower bound implied by the
% constraint that savings must be weakly positive.

tol = .01;
diffm = 1;
iterm = 0;
while diffm > tol
    iterm = iterm + 1;
    Wb_temp = zeros(sN,oN);
    for kk = 1:oN
        Wbb = zeros(sN,T);
        in_share = omega_grid(kk);
        Wbb(:,T) = pi*(sgrid'+(in_share*wim+(1-in_share)*wom*(1-taum))*(1-fo-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wim+(1-in_share)*wom)*(1-fo-fi)-cbar).^(1-s)/(1-s);
        for jj = 1:T-Tdry-1
            Wbb(:,T-jj) = pi*max(( (subplus((in_share*wim+(1-in_share)*wom*(1-taum))*(1-fo-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wbb(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wim+(1-in_share)*wom)*(1-fo-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wbb(:,T-jj+1)')');
        end
        for jj = T-Tdry:T-1
            Wbb(:,T-jj) = max( (subplus(wo*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wbb(:,T-jj+1)')');
            
        end
        Wb_temp(:,kk) = Wbb(:,1);
    end
    omega_sm = zeros(1,sN);
    Wbm = zeros(1,sN);
    for jj = 1:sN
        [Wbm(jj),oopt] = max(Wb_temp(jj,:)');
        omega_sm(jj) = omega_grid(oopt);
    end
    
    Wii = zeros(sN,T);
    in_share = 1;
    Wii(:,T) = pi*(sgrid'+(in_share*wim+(1-in_share)*wom*(1-taum))*(1-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wim+(1-in_share)*wom)*(1-fi)-cbar).^(1-s)/(1-s);
    for jj = 1:T-Tdry-1
        Wii(:,T-jj) = pi*max(( (subplus((in_share*wim+(1-in_share)*wom*(1-taum))*(1-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wii(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wim+(1-in_share)*wom)*(1-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
    end
    for jj = T-Tdry:T-1
        Wii(:,T-jj) = max( (subplus((wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
    end
    Wim = Wii(:,1)';
    
    Insidem = [Wim>=Wbm].*[Wim>=Wom];
    Bothm = [Wbm>Wim].*[Wbm>Wom];
    Outsidem = [Wom>Wbm].*[Wom>Wim];
    
    Wm = max(max(Wbm,Wim),Wom);
    
    distV = 1;
    while distV>tol
        
        Continuation = -10^16*[(Agrid'*ones(1,sN)-ones(AN,1)*sgrid)<0];
        Continuation = Continuation + Vm(1)*(.5+.5*erf(((log(Agrid(1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
        for aap = 2:AN-1
            Continuation = Continuation + Vm(aap)*((.5+.5*erf(((log(Agrid(aap))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agrid(aap-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
        end
        Continuation = Continuation + Vm(AN)*(1-(.5+.5*erf(((log(Agrid(AN-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
        
        
        [Vp,saveoptm] = max( ((1-b)*ones(AN,1)*Wm + b*Continuation)' );
        
        distV = sum(abs(Vp-Vm))/AN
        Vm = Vp;
        
    end
    
    distM = 1;
    while distM>tol
        Mp = zeros(ANf,1);
        for aa = 1:ANf
            Mp(1) = Mp(1) + Mm(aa)*(.5+.5*erf(((log(Agridf(1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
            
            for aap = 2:ANf-1
                Mp(aap) = Mp(aap) + Mm(aa)*((.5+.5*erf(((log(Agridf(aap))-mu*log(subplus(Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agridf(aap-1))-mu*log((Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
            end
            Mp(ANf) = Mp(ANf) + Mm(aa)*(1-(.5+.5*erf(((log(Agridf(ANf-1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wim)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
        end
        
        distM = sum(abs(Mp-Mm))*100
        Mm = Mp/sum(Mp);
    end
    
    Nim = zeros(1,ANf);
    InsideLaborm = zeros(ANf,1);
    OutsideLaborm = zeros(ANf,1);
    for aa = 1:ANf
        Nim(1,aa) = (Agridf(aa)-sgrid(saveoptm(aa)))/Twet/wim/(1+a/(1-a)*(Twet*wim/Px)^(eta-1));
        InsideLaborm(aa) = omega_sm(saveoptm(aa))*Bothm(saveoptm(aa))*(1-fi-fo) + Insidem(saveoptm(aa))*(1-fi);
        OutsideLaborm(aa) = (1-omega_sm(saveoptm(aa)))*Bothm(saveoptm(aa))*(1-fi-fo) + Outsidem(saveoptm(aa))*(1-fo);
    end
    
    Labor_Demandm = Nim*Mm
    Labor_Supplym = InsideLaborm'*Mm
    
    if iterm == 1
        diffm = [ [saveoptm==1]*Mm<Labor_Demandm]; % At the initial guess
    else
        
        updatem = min(.5,diffm/10);
        ExcessDemandm = Labor_Demandm-Labor_Supplym;
        diffm = abs(ExcessDemandm-1);
        
        if diffm>tol
            wim = (1-update)*wim + update*wim*(1+min(.2,max(-.2,ExcessDemandm)))
        end
    end
    
end

% Compute transition in the mean-only economy
Tranm = 20;
wimt = ones(1,Tranm)*wimL; % Transition in wages; if [savings = 0]*M > labor demand at that wage, then this is an equilibrium period-by-period
Mmt = zeros(AN,Tranm+1);
Mmt(:,1) = M;
LaborDemandmt = zeros(1,Tranm);
ZeroSavemt = zeros(1,Tranm);

% First, check if the initial guess is right
for tt = 1:Tranm
    for aa = 1:ANf
        Mmt(1,tt+1) = Mmt(1,tt+1) + + Mmt(aa,tt)*(.5+.5*erf(((log(Agridf(1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wimL)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
        for aap = 2:ANf-1
            Mmt(aap,tt+1) = Mmt(aap,tt+1) + Mmt(aa,tt)*((.5+.5*erf(((log(Agridf(aap))-mu*log(subplus(Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wimL)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agridf(aap-1))-mu*log((Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wimL)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
        end
        Mmt(ANf,tt+1) = Mmt(ANf,tt+1) + Mmt(aa,tt)*(1-(.5+.5*erf(((log(Agridf(ANf-1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptm(aa))))-mu/(eta-1)*log((1-a)/(Twet*wimL)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
    end
    Mmt(:,tt+1)=Mmt(:,tt+1)/sum(Mmt(:,tt+1));
    
    LaborDemandmt(tt) = Nim*Mmt(:,tt);
    ZeroSavemt(tt) = [saveoptm==1]*Mmt(:,tt);
end

NumZerom = sum([LaborDemandmt>ZeroSavemt]);

if NumZerom > 0
    
    wimt = wimL*ones(1,NumZerom+1);
    Vmt = ones(NumZerom+2,1)*Vm;
    Wmt = zeros(NumZerom+1,sN);
    Insidemt = zeros(NumZerom+1,sN);
    saveoptmt = ones(NumZerom+1,AN);
    diffmt = 1;
    Nimt = zeros(NumZerom+1,AN);
    LaborSupplymt = zeros(NumZerom+1,AN);
    Supplymt = zeros(NumZerom+1,1);
    
    itermt = 0
    while diffmt > .02
        itermt = itermt +1;
        for ttt = 1:NumZerom+1
            
            Wii = zeros(sN,T);
            in_share = 1;
            Wii(:,T) = pi*(sgrid'+(in_share*wimt(ttt)+(1-in_share)*wom*(1-taum))*(1-fi)-cbar).^(1-s)/(1-s) + (1-pi)*(sgrid'+(in_share*wimt(ttt)+(1-in_share)*wom)*(1-fi)-cbar).^(1-s)/(1-s);
            for jj = 1:T-Tdry-1
                Wii(:,T-jj) = pi*max(( (subplus((in_share*wimt(ttt)+(1-in_share)*wom*(1-taum))*(1-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wii(:,T-jj+1)')') + (1-pi)*max( (subplus((in_share*wimt(ttt)+(1-in_share)*wom)*(1-fi) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
            end
            for jj = T-Tdry:T-1
                Wii(:,T-jj) = max( (subplus((wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
            end
            Wimt = Wii(:,1)';
            
            Wmt(ttt,:) = max(Wimt,Wom);
            Insidemt(ttt,:) = [Wimt>=Wom];
        end
        
        if mod(itermt,5) == 1
            for ttt = 1:NumZerom+1
                Continuation = -10^16*[(Agrid'*ones(1,sN)-ones(AN,1)*sgrid)<0];
                Continuation = Continuation + Vmt(NumZerom+3-ttt,1)*(.5+.5*erf(((log(Agrid(1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wimt(NumZerom+2-ttt))^eta+a/Px^eta))-zm)/zs/sqrt(2)));
                for aap = 2:AN-1
                    Continuation = Continuation + Vmt(NumZerom+3-ttt,aap)*((.5+.5*erf(((log(Agrid(aap))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wimt(NumZerom+2-ttt))^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agrid(aap-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wimt(NumZerom+2-ttt))^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
                end
                Continuation = Continuation + Vmt(NumZerom+3-ttt,AN)*(1-(.5+.5*erf(((log(Agrid(AN-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wimt(NumZerom+2-ttt))^eta+a/Px^eta))-zm)/zs/sqrt(2))));
                
                [Vmt(NumZerom+2-ttt,:),saveoptmt(NumZerom+2-ttt,:)] = max( ((1-b)*ones(AN,1)*Wmt(NumZerom+2-ttt,:) + b*Continuation)' );
                
            end
        end
        
        for tt = 1:NumZerom+1
            Nimt(tt,:) = (Agridf-sgrid(saveoptmt(tt,:)))/Twet/wimt(tt)/(1+a/(1-a)*(Twet*wimt(tt)/Px)^(eta-1));
            LaborSupplymt(tt,:) = (1-fi)*Insidemt(tt,saveoptmt(tt,:));
        end
        
        Mmt(:,2:NumZerom+2) = zeros(AN,NumZerom+1);
        for tt = 1:NumZerom+1
            for aa = 1:ANf
                Mmt(1,tt+1) = Mmt(1,tt+1) + + Mmt(aa,tt)*(.5+.5*erf(((log(Agridf(1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptmt(tt,aa))))-mu/(eta-1)*log((1-a)/(Twet*wimt(tt))^eta+a/Px^eta))-zm)/zs/sqrt(2)));
                for aap = 2:ANf-1
                    Mmt(aap,tt+1) = Mmt(aap,tt+1) + Mmt(aa,tt)*((.5+.5*erf(((log(Agridf(aap))-mu*log(subplus(Agridf(aa)-sgrid(saveoptmt(tt,aa))))-mu/(eta-1)*log((1-a)/(Twet*wimt(tt))^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agridf(aap-1))-mu*log((Agridf(aa)-sgrid(saveoptmt(tt,aa))))-mu/(eta-1)*log((1-a)/(Twet*wimt(tt))^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
                end
                Mmt(ANf,tt+1) = Mmt(ANf,tt+1) + Mmt(aa,tt)*(1-(.5+.5*erf(((log(Agridf(ANf-1))-mu*log(subplus(Agridf(aa)-sgrid(saveoptmt(tt,aa))))-mu/(eta-1)*log((1-a)/(Twet*wimt(tt))^eta+a/Px^eta))-zm)/zs/sqrt(2))));
            end
            Mmt(:,tt+1)=Mmt(:,tt+1)/sum(Mmt(:,tt+1));
            
            LaborDemandmt(tt) = Nimt(tt,:)*Mmt(:,tt);
            ZeroSavemt(tt) = [saveoptmt(tt,:)==1]*Mmt(:,tt);
            Supplymt(tt) = LaborSupplymt(tt,:)*Mmt(:,tt);
        end
        
        
        diffmt = sum(abs((LaborDemandmt(1:length(wimt))./Supplymt(1:length(wimt))'-1).*[wimt>wimL]+[wimt==wimL].*[ZeroSavemt(1:length(wimt))<LaborDemandmt(1:length(wimt))]))/length(wimt)
        
        if diffmt>.02
            wimt = max(wimL,wimt*.95+.05*wimt.*((LaborDemandmt(1:length(wimt))./Supplymt(1:length(wimt))').^((1+rand(1,1))/20)))
        end
        
    end
end
toc


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  6. Compute change in only sd of tau
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Like the baseline scenario, this case has no risk after the bridge, so
% inside wages are pinned down by an indifference condition, no one works
% in both markets, and all individuals are indifferent over where they
% work.

wos = .7766;
taus = .0;

wis = wos*(1-fo)/(1-fi);

Wii = zeros(sN,T);
Wii(:,T) = (sgrid'+(wis)*(1-fi)-cbar).^(1-s)/(1-s) ;
for jj = 1:T-Tdry-1
    Wii(:,T-jj) = max(( (subplus((wis)*(1-fi)-cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid)).^(1-s)/(1-s) + ones(sN,1)*Wii(:,T-jj+1)')');
end
for jj = T-Tdry:T-1
    Wii(:,T-jj) = max( (subplus((wo)*(1-fo) -cbar + sgrid'*ones(1,sN) - ones(sN,1)*sgrid).^(1-s)/(1-s)  + ones(sN,1)*Wii(:,T-jj+1)')');
end
Ws = Wii(:,1)';

Vs = V;

Vdist = 10;
while Vdist >.005
    Continuation = -10^16*[(Agrid'*ones(1,sN)-ones(AN,1)*sgrid)<0];
    Continuation = Continuation + Vs(1)*(.5+.5*erf(((log(Agrid(1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
    for aap = 2:AN-1
        Continuation = Continuation + Vs(aap)*((.5+.5*erf(((log(Agrid(aap))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agrid(aap-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
    end
    Continuation = Continuation + Vs(AN)*(1-(.5+.5*erf(((log(Agrid(AN-1))-mu*log(subplus(Agrid'*ones(1,sN)-ones(AN,1)*sgrid))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
    
    
    [Vp,saveopts] = max( ((1-b)*ones(AN,1)*Ws + b*Continuation)' );
    
    Vdist = sum(abs(Vp-Vs))/AN;
    Vs = Vp;
end

Mst = zeros(AN,Tranm+1);
Mst(:,1) = M;
for tt = 1:Tranm
    for aa = 1:ANf
        Mst(1,tt+1) = Mst(1,tt+1) + + Mst(aa,tt)*(.5+.5*erf(((log(Agridf(1))-mu*log(subplus(Agridf(aa)-sgrid(saveopts(aa))))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2)));
        for aap = 2:ANf-1
            Mst(aap,tt+1) = Mst(aap,tt+1) + Mst(aa,tt)*((.5+.5*erf(((log(Agridf(aap))-mu*log(subplus(Agridf(aa)-sgrid(saveopts(aa))))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2))) - (.5+.5*erf(((log(Agridf(aap-1))-mu*log((Agridf(aa)-sgrid(saveopts(aa))))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2))) );
        end
        Mst(ANf,tt+1) = Mst(ANf,tt+1) + Mst(aa,tt)*(1-(.5+.5*erf(((log(Agridf(ANf-1))-mu*log(subplus(Agridf(aa)-sgrid(saveopts(aa))))-mu/(eta-1)*log((1-a)/(Twet*wis)^eta+a/Px^eta))-zm)/zs/sqrt(2))));
    end
    Mst(:,tt+1)=Mst(:,tt+1)/sum(Mst(:,tt+1));
    
end

Nis = (Agridf-sgrid(saveopts))/Twet/wis/(1+a/(1-a)*(Twet*wis/Px)^(eta-1));

DecreaseStorage = (sgrid(saveoptf2)*Mtt(:,1)-sgrid(saveoptf)*M)/(Consumption_trans(1)-Consumption_pre)
ExtensiveMargin = (Twet*wi*(Ni2*M - Inside_Labor_pre'*M) + Twet*wo*(1-pi+pi*(1-tau1))*((1-fo)*(1-Ni2*M/(1-fi))-Outside_Labor_pre*M))/(Consumption_trans(1)-Consumption_pre)
EarningsMargin = (Twet*(wi2-wi)*(Ni2*M) + Twet*(wo-(1-pi+pi*(1-tau1))*wo)*((1-fo)*(1-Ni2*M/(1-fi))))/(Consumption_trans(1)-Consumption_pre)

Consumption_trans(1,Transition) = (sgrid(saveoptf2)+Twet*wi2*(1-fi)+Tdry*wo*(1-fo))*Mtt(:,Transition);
Consumptionmt = zeros(1,NumZerom+1);
for tt = 1:NumZerom+1
    Consumptionmt(tt) = (sgrid(saveoptmt(tt,:))+Twet*(InsideLaborm*wimt(tt) + wom*OutsideLaborm*(1-pi+pi*(1-taum)))'+Tdry*wo*(1-fo))*Mmt(:,tt);
end
Consumptionm = (sgrid(saveoptm)+Twet*(InsideLaborm*wim + wom*OutsideLaborm*(1-pi+pi*(1-taum)))'+Tdry*wo*(1-fo))*Mm;
Consumptions = (sgrid(saveopts)+Twet*wis*(1-fi)+Tdry*wo*(1-fo))*Mst(:,Tranm);
Consumptionst = (sgrid(saveopts)+Twet*wis*(1-fi)+Tdry*wo*(1-fo))*Mst;


DecreaseStoragem = (sgrid(saveoptmt(1,:))*Mmt(:,1)-sgrid(saveoptf)*M)/(Consumptionmt(1)-Consumption_pre)
ExtensiveMarginm = (Twet*wi*(InsideLaborm'*M - Inside_Labor_pre'*M) + Twet*wo*(1-pi+pi*(1-tau1))*(OutsideLaborm'*Mmt(:,1)-Outside_Labor_pre*M))/(Consumptionmt(1)-Consumption_pre)
EarningsMarginm = (Twet*(wimt(1)-wi)*(InsideLaborm'*M) + Twet*((1-pi+pi*(1-taum))*wom-wo*(1-pi+pi*(1-tau1)))*(OutsideLaborm'*M))/(Consumptionmt(1)-Consumption_pre)

DecreaseStorages = (sgrid(saveopts)*M-sgrid(saveoptf)*M)/(Consumptionst(1)-Consumption_pre)
ExtensiveMargins = (Twet*wi*(Nis*M - Inside_Labor_pre'*M) + Twet*wo*(1-pi+pi*(1-tau1))*((1-fo)*(1-Nis*M/(1-fi))-Outside_Labor_pre*M))/(Consumptionst(1)-Consumption_pre)
EarningsMargins = (Twet*(wis-wi)*(Nis*M) + Twet*(wos-(1-pi+pi*(1-tau1))*wo)*((1-fo)*(1-Nis*M/(1-fi))))/(Consumptionst(1)-Consumption_pre)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  7. Compute model output tables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PARAMETERS_TABLE = [b mu a s zm zs eta fo fi]'

RESULTS_TABLE = [ log((V2*M)/(V*M))/(1-s)       log((V2*M)/(V2*Mtt(:,Transition)))/(1-s)    log(Consumption_trans(1,Transition)/Consumption_pre)    log(((V2*Mtt(:,Transition))/Consumption_trans(1,Transition)^(1-s)) /((V*M)/Consumption_pre^(1-s)))/(1-s)    wi2/wi-1    (sum((saveoptf2)*Mtt(:,1:2)/2)/(saveopt*M))-1 (((1-fo)*wo*(1-Ni2*M/(1-fi)))/(Outside_Earn'*M)-1) (1-fo)*wo/((Inside_Earn+Outside_Earn)'*M)-1;
    log((Vmt(1,:)*M)/(V*M))/(1-s) log((Vmt(1,:)*M)/(Vm*Mm))/(1-s)             log(Consumptionm/Consumption_pre)                       log(((Vm*Mm)/Consumptionm^(1-s)) /((V*M)/Consumption_pre^(1-s)))/(1-s)                                      wim/wi-1    (sum((saveoptm)*Mmt(:,1:2)/2)/(saveopt*M))-1  (((1-fo)*wom*(1-Nimt(1,:)*M/(1-fi)))/(Outside_Earn'*M)-1) ((InsideLaborm*wimt(1)+OutsideLaborm*wom)'*M)/((Inside_Earn+Outside_Earn)'*M)-1;
    log((Vs*M)/(V*M))/(1-s)       log((Vs*M)/(Vs*Mst(:,Tranm)))/(1-s)         log(Consumptions/Consumption_pre)                       log(((Vs*Mst(:,Tranm))/Consumptions^(1-s)) /((V*M)/Consumption_pre^(1-s)))/(1-s)                            wis/wi-1    (sum((saveopts)*Mst(:,1:2)/2)/(saveopt*M))-1  (((1-fo)*wos*(1-Nis*M/(1-fi)))/(Outside_Earn'*M)-1) (1-fo)*wos/((Inside_Earn+Outside_Earn)'*M)-1 ]'

Cons_Transition = [0 Consumption_trans(1,1:10)./Consumption_pre-1];
plot([0:10],Cons_Transition)

MODELfinal = [Excess_Labor_Demand SD_Log_Harvest_model Intm_Spending MODELu(1) MODELu(2)];
% save FinalResults_20JAN20